/* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */

#include "AudioRecordSourceFactory.h"

#include "AudioJACKSource.h"
#include "AudioPortAudioSource.h"

#include <iostream>

AudioCallbackRecordSource *
AudioRecordSourceFactory::createCallbackRecordSource(AudioCallbackRecordTarget *target)
{
    AudioCallbackRecordSource *source = 0;

#ifdef HAVE_JACK
    source = new AudioJACKSource(target);
    if (source->isOK()) return source;
    else {
	std::cerr << "WARNING: AudioRecordSourceFactory::createCallbackRecordSource: Failed to open JACK source" << std::endl;
	delete source;
    }
#endif

#ifdef HAVE_PORTAUDIO
    source = new AudioPortAudioSource(target);
    if (source->isOK()) return source;
    else {
	std::cerr << "WARNING: AudioRecordSourceFactory::createCallbackRecordSource: Failed to open PortAudio source" << std::endl;
	delete source;
    }
#endif

    std::cerr << "WARNING: AudioRecordSourceFactory::createCallbackRecordSource: No suitable sources available" << std::endl;
    return 0;
}


