/* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */

#include <vamp-hostsdk/PluginHostAdapter.h>

#include "audioio/BufferingAudioCallbackRecordTarget.h"
#include "audioio/AudioRecordSourceFactory.h"

#include <iostream>
#include <sndfile.h>

// include "audioio/System.h"

#include "Processor.h"
#include "SimpleXMLRuleLoader.h"

#include <cmath>
#include <cassert>

#include <QApplication>
#include <QFont>
#include <QString>
#include <QStringList>
#include <QPushButton>

using std::cout;
using std::cerr;
using std::endl;
using std::string;
using std::vector;


int main(int argc, char **argv)
{
    QApplication application(argc, argv);

    QStringList args = application.arguments();

    QString fileName;
    for (QStringList::iterator i = args.begin(); i != args.end(); ++i) {
        if (i == args.begin()) continue;
        if (i->startsWith('-')) continue;
        fileName = *i;
        break;
    }

    if (fileName == "") {
        std::cerr << "Usage: " << args.begin()->toStdString() << " <ruleset.xml>" << std::endl;
        return 2;
    }

    QFont fn = application.font();
    fn.setPointSize(fn.pointSize() + 3);
    application.setFont(fn);

    QPushButton *button = new QPushButton("Quit");
    QObject::connect(button, SIGNAL(clicked()), 
                     &application, SLOT(closeAllWindows()));
    
    BufferingAudioCallbackRecordTarget *target =
        new BufferingAudioCallbackRecordTarget();

    AudioCallbackRecordSource *source =
        AudioRecordSourceFactory::createCallbackRecordSource(target);
    if (!source) {
        std::cerr << "ERROR: Failed to create audio record source" << std::endl;
        return 1;
    }

    Processor processor(target);

    SimpleXMLRuleLoader loader;
    if (!loader.loadFile(processor, fileName)) {
        std::cerr << "ERROR: Failed to load XML ruleset file \"" << fileName.toStdString() << "\"" << std::endl;
        return 1;
    }

    button->show();
    processor.start();

    application.exec();
}

